﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace Shangji
{
    class ImageGenerator
    {
        private readonly string _姓名;

        private readonly int _月份;

        private readonly int _商机总数;

        private readonly int _上月商机总数;

        private readonly int _季度商机数量;

        private readonly int _预计成单商机数量;

        private readonly int _拜访率; 

        private byte[] _imageData;

        public byte[] ImageData => _imageData;

        public ImageSource Image
        {
            get
            {
                var bitmapImage = new BitmapImage();
                bitmapImage.BeginInit();
                bitmapImage.StreamSource = new MemoryStream(_imageData);
                bitmapImage.EndInit();
                return bitmapImage;
            }
        }

        public ImageGenerator(string 姓名, int 月份, int 商机总数, int 上月商机总数, int 季度商机数量, int 预计成单商机数量, int 拜访率)
        {
            _姓名 = 姓名;
            _月份 = 月份;
            _商机总数 = 商机总数;
            _上月商机总数 = 上月商机总数;
            _季度商机数量 = 季度商机数量;
            _预计成单商机数量 = 预计成单商机数量;
            _拜访率 = 拜访率;
            CreateImage();
        }

        private void CreateImage()
        {
            var 季度 = Math.Ceiling(4 * ((float)_月份 / 12));
            var 预计成交商机数量占比 = (float)_预计成单商机数量 / _季度商机数量 * 100;
            var 绿区文字 = new List<string>();
            var 黄区文字 = new List<string>();
            var 红区文字 = new List<string>();

            var 商机总数趋势 = $"商机总数：{_月份}月 {_商机总数}";
            if (_月份 - 1 > 0)
            {
                商机总数趋势 = $"商机总数：{_月份 - 1}月 {_上月商机总数} → {_月份}月 {_商机总数}";
            }
            var 月度客户拜访率 = $"月度客户拜访率：{_拜访率}%";
            var 季度商机数量 = $"Q{季度}商机数量：{_季度商机数量}";
            var 季度预计成单商机数量 = $"Q{季度}预计成单商机数量：{_预计成单商机数量}";
            var 成单率 = $"成单率：{(int)((float)_预计成单商机数量 / _季度商机数量 * 100)}% ";

            var nameIndex = 0;

            if (_商机总数 > 80)
            {
                绿区文字.Add(商机总数趋势);
                绿区文字.Add(月度客户拜访率);

            }
            else if (_商机总数 < 50)
            {
                nameIndex = 2;
                红区文字.Add(商机总数趋势);
                红区文字.Add(月度客户拜访率);
            }
            else
            {
                nameIndex = 1;
                黄区文字.Add(商机总数趋势);
                黄区文字.Add(月度客户拜访率);
            }

            if(预计成交商机数量占比 > 20)
            {
                绿区文字.Add(季度商机数量);
                绿区文字.Add(季度预计成单商机数量);
                绿区文字.Add(成单率);
            }
            else if(预计成交商机数量占比 < 10)
            {
                红区文字.Add(季度商机数量);
                红区文字.Add(季度预计成单商机数量);
                红区文字.Add(成单率);
            }
            else
            {
                黄区文字.Add(季度商机数量);
                黄区文字.Add(季度预计成单商机数量);
                黄区文字.Add(成单率);
            }

            Assembly assembly = Assembly.GetExecutingAssembly();
            using (var stream = assembly.GetManifestResourceStream($"{GetType().Namespace}.Template.png"))
            {
                var font = new Font(new System.Drawing.FontFamily("微软雅黑"), 20,FontStyle.Bold, GraphicsUnit.Pixel);
                using (var bitmap = (Bitmap)System.Drawing.Image.FromStream(stream))
                {
                    //每个区 宽度330 开始位置高度350
                    using (var g = Graphics.FromImage(bitmap))
                    {
                        //画名字
                        var nameSize = g.MeasureString(_姓名, font);
                        var nameTop = 320;
                        if (nameIndex == 0)
                        {
                            var left = (330 - nameSize.Width) / 2;
                            g.DrawString(_姓名, font, System.Drawing.Brushes.White, new PointF(left, nameTop));
                        }
                        if(nameIndex == 1)
                        {
                            var left = 330 + (360 - nameSize.Width) / 2;
                            g.DrawString(_姓名, font, System.Drawing.Brushes.Black, new PointF(left, nameTop));
                        }
                        if(nameIndex == 2)
                        {
                            var left = 690 + (330 - nameSize.Width) / 2;
                            g.DrawString(_姓名, font, System.Drawing.Brushes.White, new PointF(left, nameTop));
                        }

                        //绿区
                        var i = 0;
                        foreach (var text in 绿区文字)
                        {
                            var textSize = g.MeasureString(text, font);
                            var top = 410 + i * textSize.Height + 2;
                            var left = (330 - textSize.Width) / 2;
                            g.DrawString(text, font, System.Drawing.Brushes.White, new PointF(left, top));
                            i++;
                        }

                        //黄区
                        i = 0;
                        foreach (var text in 黄区文字)
                        {
                            var textSize = g.MeasureString(text, font);
                            var top = 410 + i * textSize.Height + 2;
                            var left = 330 + (360 - textSize.Width) / 2;
                            g.DrawString(text, font, System.Drawing.Brushes.Black, new PointF(left, top));
                            i++;
                        }

                        //红区
                        i = 0;
                        foreach (var text in 红区文字)
                        {
                            var textSize = g.MeasureString(text, font);
                            var top = 410 + i * textSize.Height + 2;
                            var left = 690 + (330 - textSize.Width) / 2;
                            g.DrawString(text, font, System.Drawing.Brushes.White, new PointF(left, top));
                            i++;
                        }
                    }

                    using (var imageDataStream = new MemoryStream())
                    {
                        using (var newBitmap = ResizeImage(bitmap, 0.7f))
                        {
                            if (newBitmap != null)
                            {
                                newBitmap.Save(imageDataStream, System.Drawing.Imaging.ImageFormat.Png);
                                _imageData = imageDataStream.ToArray();
                            }
                        }
                    }
                }

            }
        }

        public static Bitmap ResizeImage(Bitmap source, float scale)
        {
            //return source;
            try
            {
                var newWidth = (int)(source.Width * scale);
                var newHeight = (int)(source.Height * scale);
                var newBitmap = new Bitmap(source, newWidth, newHeight);
                return newBitmap;
            }
            catch
            {
                return null;
            }
        }

    }
}
