﻿using NPOI.SS.UserModel;
using System;
using System.Diagnostics;
using System.IO;
using System.Net.Mail;
using System.Threading.Tasks;
using System.Windows;

namespace Shangji
{

    public class MailTask
    {
        public string Name { get; }

        public string Address { get; }

        public string Title { get; }

        public string Sender { get; }

        public string Content { get; }

        public string[] CCList { get; }

        public MailTask(string name, string address, string title, string sender, string content, string[] cCList)
        {
            Name = name;
            Address = address;
            Title = title;
            Sender = sender;
            Content = content;
            CCList = cCList;
        }

    }

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            MailHost.Text = "host.vinno.com";
            var shangjiFile = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "shangji.xlsx");
            if (!File.Exists(shangjiFile))
            {
                MessageBox.Show("未找到商机文件，请将shangji.xlsx放入程序文件夹再执行。");
                Process.GetCurrentProcess().Kill();
            }
        }

        private void OnSendClick(object sender, RoutedEventArgs e)
        {
            Progress.Value = 0;
            var mailHost = MailHost.Text;
            var senderAddress = SenderMail.Text;
            var senderPassword = SenderPassword.Password;
            var senderDisplayName = SenderDisplayName.Text;
            MailHost.IsEnabled = false;
            SenderMail.IsEnabled = false;
            SenderPassword.IsEnabled = false;
            SenderDisplayName.IsEnabled = false;
            SendButton.IsEnabled = false;
            Task.Run(() =>
            {
                try
                {
                    var shangjiFile = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "shangji.xlsx");
                    using (FileStream shangjiStream = new FileStream(shangjiFile, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
                    {
                        IWorkbook workbook = WorkbookFactory.Create(shangjiStream);
                        try
                        {
                            ISheet sheet = workbook.GetSheetAt(0);
                            var startRow = 1;
                            var endRow = sheet.LastRowNum;
                            var totalCount = endRow - startRow + 1;
                            var handledTask = 0;
                            for (var i = startRow; i <= endRow; i++)
                            {
                                var row = sheet.GetRow(i);
                                if (row != null)
                                {
                                    var 姓名 = row.Cells[0].StringCellValue;
                                    var 邮箱 = row.Cells[1].StringCellValue;
                                    var 抄送列表 = row.Cells[2].StringCellValue;
                                    var 月份 = (int)row.Cells[3].NumericCellValue;
                                    var 本月商机总数 = (int)row.Cells[4].NumericCellValue;
                                    var 上月商机总数 = (int)row.Cells[5].NumericCellValue;
                                    var 本季度商机数量 = (int)row.Cells[6].NumericCellValue;
                                    var 本季度预计成单商机数量 = (int)row.Cells[7].NumericCellValue;
                                    var 月度客户拜访率 = (int)row.Cells[8].NumericCellValue;
                                    var 抄送人 = 抄送列表.Split(';');

                                    var imageGenerator = new ImageGenerator(姓名, 月份, 本月商机总数, 上月商机总数, 本季度商机数量, 本季度预计成单商机数量, 月度客户拜访率);
                                    var imageData = imageGenerator.ImageData;
                                    var mailContent = $"<BODY style=\"MARGIN: 4px\"><DIV><IMG width=\"716\" height=\"415\" src=\"data:image/png;base64,{Convert.ToBase64String(imageData)}\"> </IMG></DIV><DIV style=\"font-size:16px\">---This email was generated by the Shangji tool created by Justin:)</DIV></BODY>";
                                    var task = new MailTask(姓名, 邮箱, $"{姓名} 截止至{月份}月商机报告", senderDisplayName, mailContent, 抄送人);
                                    SendMail(mailHost, senderAddress, senderPassword, task);
                                    handledTask++;
                                    var progress = (int)((float)handledTask / totalCount * 100);
                                    Dispatcher.Invoke(() =>
                                    {
                                        Progress.Value = progress;
                                    });
                                }
                            }
                        }
                        finally
                        {
                            workbook.Close();
                        }
                    }

                }
                catch (Exception ex)
                {
                    WriteLog($"读取商机统计并发送邮件失败,原因：{ex.Message}");
                }
                finally
                {
                    Dispatcher.Invoke(() =>
                    {
                        MailHost.IsEnabled = true;
                        SenderMail.IsEnabled = true;
                        SenderPassword.IsEnabled = true;
                        SenderDisplayName.IsEnabled = true;
                        SendButton.IsEnabled = true;
                    });
                }
            });
        }

        private void SendMail(string mailHost, string senderAddress, string senderPassword, MailTask mailTask)
        {
            WriteLog($"正在发送邮件给 {mailTask.Name}({mailTask.Address})...");
            var client = new SmtpClient(mailHost);
            client.UseDefaultCredentials = true;
            client.Credentials = new System.Net.NetworkCredential(senderAddress, senderPassword);
            var message = new MailMessage();
            message.From = new MailAddress(senderAddress, mailTask.Sender);
            message.To.Add(new MailAddress(mailTask.Address, mailTask.Address));
            if(mailTask.CCList != null)
            {
                foreach(var cc in mailTask.CCList)
                {
                    message.CC.Add(cc);
                }
            }
            message.Subject = mailTask.Title;
            message.IsBodyHtml = true;
            message.Body = mailTask.Content;
            try
            {
                client.Send(message);
                WriteLog($"发送邮件给 {mailTask.Name}({mailTask.Address}) 完成。");
            }
            catch (Exception ex)
            {
                WriteLog($"邮件发送失败,原因：{ex.Message}");
            }
        }

        private void WriteLog(string message)
        {
            Dispatcher.Invoke(() =>
                {
                    LogList.Items.Add(message);
                    LogScrollViewer.ScrollToEnd();
                }
            );
        }
    }
}
